searchData={"items":[{"type":"behaviour","title":"Annealing","doc":"A generic implementation of the Simulated Annealing optimization algorithm.\n\n  Simulated Annealing is a probabilistic technique for approximating the global optimum\n  of a given function. It is often used to solve combinatorial optimization problems\n  like the Traveling Salesman Problem, job scheduling, or circuit design.","ref":"Annealing.html"},{"type":"behaviour","title":"Strategy Behaviour - Annealing","doc":"To use this module, you must implement a strategy module that defines the problem-specific\n  parts of the algorithm. The strategy module must implement the following callbacks:\n\n    * `init/2` - Initialize the solution state\n    * `calculate_energy/1` - Calculate the \"energy\" (cost) of a solution\n    * `get_neighbor/1` - Generate a neighboring solution","ref":"Annealing.html#module-strategy-behaviour"},{"type":"behaviour","title":"Example - Annealing","doc":"defmodule TravelingSalesmanStrategy do\n        @behaviour Annealing\n\n        @impl true\n        def init(cities, _opts), do: {:ok, cities}\n\n        @impl true\n        def calculate_energy(route) do\n          route\n          |> Enum.chunk_every(2, 1, :wrap)\n          |> Enum.map(fn [{x1, y1}, {x2, y2}] ->\n            :math.sqrt(:math.pow(x2 - x1, 2) + :math.pow(y2 - y1, 2))\n          end)\n          |> Enum.sum()\n        end\n\n        @impl true\n        def get_neighbor(route) do\n          i = :rand.uniform(length(route)) - 1\n          j = :rand.uniform(length(route)) - 1\n\n          List.replace_at(\n            List.replace_at(route, i, Enum.at(route, j)),\n            j,\n            Enum.at(route, i)\n          )\n        end\n      end\n\n      # Using the strategy\n      cities = [{0, 0}, {1, 5}, {2, 2}, {3, 3}]\n      {:ok, optimal_route} = Annealing.run(TravelingSalesmanStrategy, cities)","ref":"Annealing.html#module-example"},{"type":"behaviour","title":"Configuration Options - Annealing","doc":"The following options can be passed to `run/3`:\n\n    * `:temperature` - Initial temperature (default: 100.0)\n    * `:cooldown` - Cooling rate per iteration (default: 0.004)\n    * `:break_energy` - Target energy level to stop at (default: 0.0)\n    * `:max_iterations` - Maximum number of iterations (default: :infinity)\n    * `:name` - Optional name for the annealing process (default: nil)","ref":"Annealing.html#module-configuration-options"},{"type":"behaviour","title":"Algorithm Details - Annealing","doc":"The algorithm works by iteratively:\n  1. Generating a neighbor solution\n  2. Calculating the energy difference (ΔE) between current and neighbor\n  3. Accepting the neighbor if it's better (ΔE < 0)\n  4. Accepting worse solutions with probability P(ΔE) = e^(-ΔE/T)\n  5. Decreasing temperature according to the cooling schedule\n\n  The temperature starts high (accepting many worse solutions) and gradually\n  decreases, causing the algorithm to become more selective over time. This\n  combination of exploration and exploitation helps escape local optima.","ref":"Annealing.html#module-algorithm-details"},{"type":"callback","title":"Annealing.calculate_energy/1","doc":"","ref":"Annealing.html#c:calculate_energy/1"},{"type":"callback","title":"Annealing.generate_neighbor/1","doc":"","ref":"Annealing.html#c:generate_neighbor/1"},{"type":"function","title":"Annealing.run/3","doc":"","ref":"Annealing.html#run/3"},{"type":"type","title":"Annealing.items/0","doc":"","ref":"Annealing.html#t:items/0"},{"type":"type","title":"Annealing.run_option/0","doc":"","ref":"Annealing.html#t:run_option/0"},{"type":"type","title":"Annealing.run_options/0","doc":"","ref":"Annealing.html#t:run_options/0"},{"type":"type","title":"Annealing.state/0","doc":"","ref":"Annealing.html#t:state/0"},{"type":"module","title":"Annealing.Controller","doc":"Controls execution of simulated annealing.","ref":"Annealing.Controller.html"},{"type":"function","title":"Annealing.Controller.finish_annealing/1","doc":"","ref":"Annealing.Controller.html#finish_annealing/1"},{"type":"function","title":"Annealing.Controller.perform_annealing/1","doc":"","ref":"Annealing.Controller.html#perform_annealing/1"},{"type":"type","title":"Annealing.Controller.items/0","doc":"","ref":"Annealing.Controller.html#t:items/0"},{"type":"type","title":"Annealing.Controller.iteration_number/0","doc":"","ref":"Annealing.Controller.html#t:iteration_number/0"},{"type":"type","title":"Annealing.Controller.state/0","doc":"","ref":"Annealing.Controller.html#t:state/0"},{"type":"module","title":"Annealing.Controller.Acceptance","doc":"Contains function clauses that determine solution acceptance.","ref":"Annealing.Controller.Acceptance.html"},{"type":"function","title":"Annealing.Controller.Acceptance.choose_best_material/2","doc":"","ref":"Annealing.Controller.Acceptance.html#choose_best_material/2"},{"type":"function","title":"Annealing.Controller.Acceptance.metropolis_probability/2","doc":"Acceptance probability calculting using Metropolis criteria","ref":"Annealing.Controller.Acceptance.html#metropolis_probability/2"},{"type":"function","title":"Annealing.Controller.Acceptance.should_accept?/2","doc":"","ref":"Annealing.Controller.Acceptance.html#should_accept?/2"},{"type":"type","title":"Annealing.Controller.Acceptance.annealing_state/0","doc":"","ref":"Annealing.Controller.Acceptance.html#t:annealing_state/0"},{"type":"type","title":"Annealing.Controller.Acceptance.material/0","doc":"","ref":"Annealing.Controller.Acceptance.html#t:material/0"},{"type":"module","title":"Annealing.Controller.Cooling","doc":"Contains function clauses that cool material state.","ref":"Annealing.Controller.Cooling.html"},{"type":"function","title":"Annealing.Controller.Cooling.cooldown/1","doc":"Allows different cooling strategies to be used:\nlinear, exponential, logarithmic, or a custom function.","ref":"Annealing.Controller.Cooling.html#cooldown/1"},{"type":"type","title":"Annealing.Controller.Cooling.annealing_state/0","doc":"","ref":"Annealing.Controller.Cooling.html#t:annealing_state/0"},{"type":"type","title":"Annealing.Controller.Cooling.cooling_strategy/0","doc":"","ref":"Annealing.Controller.Cooling.html#t:cooling_strategy/0"},{"type":"module","title":"Annealing.Material","doc":"A struct to hold the volatile state of each step of the annealing algorithm.","ref":"Annealing.Material.html"},{"type":"type","title":"Annealing.Material.item_list/0","doc":"","ref":"Annealing.Material.html#t:item_list/0"},{"type":"type","title":"Annealing.Material.t/0","doc":"","ref":"Annealing.Material.html#t:t/0"},{"type":"module","title":"Annealing.Regime","doc":"A struct to hold details about the overall annealing process.","ref":"Annealing.Regime.html"},{"type":"function","title":"Annealing.Regime.iterations/1","doc":"","ref":"Annealing.Regime.html#iterations/1"},{"type":"type","title":"Annealing.Regime.t/0","doc":"","ref":"Annealing.Regime.html#t:t/0"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}